#ifndef __CXXTLS__DATETOOLS_H_INCLUDED_p
#define __CXXTLS__DATETOOLS_H_INCLUDED_p
//
// Copyright 2002-2015, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure may be covered by the
// same copyright.  Some of them may be part of the GNU source distribution
// and you must obey the GPL copyright for any such files.

#include <time.h>
#include <string>


namespace cxxtls     {
namespace DateTools  {



    struct DateFormatter
        /// a functor object that formats dates itno strings.
    {
        std::string format_;

        DateFormatter(std::string const &format)
        :  format_(format)
        {
            /// Construct a date formatter object given a string that defines the
            /// the date format.  The format string is the same as the unix date
            /// command accepts:  %Y, for examples is the 4 digit year.  
            ///
            /// The following formats are @b not supported:
            ///
            ///   @li %g   (see %G)
            ///   @li %G   (ISO monday of the current week)
            ///   @li %N   (nanoseconds)
            ///   @li %U   (week of year)
            ///   @li %V   (ISO week of year)
            ///   @li %W   (week number of the year
            ///   @li %::z
            ///   @li %:::z
            
        }

        virtual std::string operator() (struct tm const *tim, time_t   secs); 
            /// a functor method that converts an "exploded" time into a string
            /// using the format specified to the constructor.
            ///
            /// @param tim  The exploded time to be converted to the string
            /// @param secs The UTC time in seconds (since jan 1970)
            ///
            /// Note that the tim value is usually obtained by calling time()
            /// then using one of the following functions to explode it:
            ///
            /// @li localtime()    
            /// @li gmtime()       
            /// @li localtime_r()
            /// @li localtime_s()
            ///
            /// Depending on the OS and the desired time references.
            ///
            /// This function is thread safe.

        std::string operator() (time_t t, bool gm = false)
            /// a functor method that converts a time into a string
            /// using the format specified to the constructor.
            ///
            /// @param t  The time value to be converted
            ///
            /// @param gm A flag indicating that greenwich mean time time
            ///           (aka UTC) reference is needed.   False means local
            ///           time reference.
            /// 
            /// This function is thread safe.
            ///
            /// It exists to simplify the interface to the virtual method
            /// defined above.
        {
#ifdef _MSC_VER
            struct tm tmp;

            if(gm)
            {
               if( 0 == gmtime_s(&tmp, &t) )
               {
                   return (*this)(&tmp, t);  // call the virtual method
               }
            }
            else
            {
               if( 0 == localtime_s(&tmp, &t) )
               {
                   return (*this)(&tmp, t);  // call the virtual method
               }
            }

            return "";
#else
            struct tm tmp;

            if(gm)
            {
               if( gmtime_r(&t, &tmp) )
               {
                   return (*this)(&tmp, t);  // call the virtual method
               }
            }
            else
            {
               if( localtime_r(&t, &tmp) )
               {
                   return (*this)(&tmp, t);  // call the virtual method
               }
            }

            return "";
#endif
        }

        std::string operator() (bool gm = false)
            /// a functor method that converts the current time into a string
            /// using the format specified to the constructor.
            ///
            /// @param gm A flag indicating that greenwich mean time time
            ///           (aka UTC) reference is needed.   False means local
            ///           time reference.
            /// 
            /// This function is thread safe.
            ///
            /// It exists to simplify the interface to the virtual method
            /// defined above.
        {
            time_t now = 0;

            time(&now);

            return (*this)(now, gm);
        }

         
    };  // class cxxtls::DateTools::DateFormatter


    char const *dayAbbrev(int day);
        ///<  Given an integer day of the week, return the 3 character abbreviation thereof.
        ///
        ///  @param[in] day  An integer between 0 and 6.  0 is sunday.
        ///
        ///  @return a 3 character abbreviation of the day name.

    char const *dayFull(int day);
        ///<  given an integer day, return the full name of the specified day of the week.
        ///
        ///  @param[in] day An integer between 0 and 6. 0 is sunday.
        ///
        ///  @return The mixed case full name of the day of the week.

    char const *monthAbbrev(int month);
        ///<  given an integer month, return the 3 letter abbreviation of the month name.
        ///
        ///  @param[in] month  An integer between 0 and 11.  0 is January.
        ///
        ///  @return a 3 character abbreviation for the month name.

    char const *monthFull(int month);
        ///<  given an integer month, return the full name of the month.
        ///
        ///  @param[in] month An integer between 0 and 11.  0 is january.
        ///
        ///  @return The mixed case full name of the month.





} // namespace cxxtls::DateTools
} // namespace cxxtls

#endif
